@@ -53,6 +53,10 @@ typedef struct {
 
 	/* Whether this item was selected before rubberbanding. */
 	eel_boolean_bit was_selected_before_rubberband : 1;
+
+	/* Whether a monitor was set on this icon. */
+	eel_boolean_bit is_monitored : 1;
+
 } NautilusIcon;
 
 
@@ -74,7 +78,6 @@ typedef struct {
 typedef enum {
 	DRAG_STATE_INITIAL,
 	DRAG_STATE_MOVE_OR_COPY,
-	DRAG_STATE_MOVE_COPY_OR_MENU,
 	DRAG_STATE_STRETCH
 } DragState;
 
@@ -157,6 +160,9 @@ struct NautilusIconContainerDetails {
 	StretchState stretch_start;
 	int context_menu_timeout_id;
 
+	gboolean icon_selected_on_button_down;
+	NautilusIcon *double_click_icon[2]; /* Both clicks in a double click need to be on the same icon */
+
 	/* Renaming Details */
 	gboolean renaming;
 	GtkWidget *rename_widget;	/* Editable text item */
@@ -171,6 +177,9 @@ struct NautilusIconContainerDetails {
 	/* Idle handler for stretch code */
 	guint stretch_idle_id;
 
+	/* Align idle id */
+	guint align_idle_id;
+
 	/* DnD info. */
 	NautilusIconDndInfo *dnd_info;
 
@@ -197,6 +206,9 @@ struct NautilusIconContainerDetails {
 	 */
 	Axis arrow_key_axis;
 	int arrow_key_start;
+	int arrow_key_start_x;
+	int arrow_key_start_y;
+	GtkDirectionType arrow_key_direction;
 
 	/* Mode settings. */
 	gboolean single_click_mode;
@@ -206,7 +218,10 @@ struct NautilusIconContainerDetails {
 	/* Layout mode */
 	NautilusIconLayoutMode layout_mode;
 
-	/* Set to TRUE after first allocation has been done */
+	/* Should the container keep icons aligned to a grid */
+	gboolean keep_aligned;
+
+        /* Set to TRUE after first allocation has been done */
 	gboolean has_been_allocated;
 	
 	/* Is the container fixed or resizable */
@@ -243,6 +258,7 @@ void          nautilus_icon_container_move_icon                   (NautilusIconC
 								   double                 scale_x,
 								   double                 scale_y,
 								   gboolean               raise,
+								   gboolean               snap,
 								   gboolean		  update_position);
 void          nautilus_icon_container_select_list_unselect_others (NautilusIconContainer *container,
 								   GList                 *icons);
