@@ -32,7 +32,7 @@
 #include "nautilus-mime-actions.h"
 #include "nautilus-program-choosing.h"
 #include "nautilus-view-identifier.h"
-#include "egg-screen-help.h"
+#include <libegg/egg-screen-help.h>
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -75,7 +75,7 @@ typedef enum {
 	PROGRAM_IN_PREFERRED_LIST_FOR_FILE,
 	PROGRAM_DEFAULT_FOR_SUPERTYPE,
 	PROGRAM_DEFAULT_FOR_TYPE,
-	PROGRAM_DEFAULT_FOR_FILE,
+	PROGRAM_DEFAULT_FOR_FILE
 } ProgramFileStatus;
 
 typedef struct {
@@ -181,9 +181,8 @@ help_cb (GtkWidget *button, NautilusProgramChooser *program_chooser)
 		break;
 	}
 
-	egg_screen_help_display_desktop (
-			gtk_window_get_screen (GTK_WINDOW (program_chooser)),
-			NULL, "user-guide", "wgosnautilus.xml", section, &error);
+	egg_help_display_desktop_on_screen (NULL, "user-guide", "wgosnautilus.xml", section,
+					    gtk_window_get_screen (GTK_WINDOW (program_chooser)), &error);
 
 	if (error) {
 		GtkWidget *err_dialog;
@@ -441,7 +440,7 @@ repopulate_program_list (NautilusProgramChooser *program_chooser)
 	
 
 	programs = type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT
-		? nautilus_mime_get_all_components_for_file_extended (program_chooser->details->file, "(NOT nautilus:property_page_name.defined)")
+		? nautilus_mime_get_all_components_for_file_extended (program_chooser->details->file, "NOT nautilus:property_page_name.defined()")
 		: nautilus_mime_get_all_applications_for_file (program_chooser->details->file);
 
 	list_store = program_chooser->details->list_store;
@@ -1406,6 +1405,7 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
 	/* Create and setup tree view */
 	create_and_set_up_tree_view (program_chooser);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), program_chooser->details->tree_view);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (program_chooser->details->prompt_label), program_chooser->details->tree_view);
 
 	/* Framed area with selection-specific details */
 	program_chooser->details->frame = gtk_frame_new (NULL);
@@ -1427,6 +1427,7 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
   	gtk_box_pack_end (GTK_BOX (framed_hbox), change_button_holder, FALSE, FALSE, 0);
 
   	change_button = gtk_button_new_with_mnemonic (_("_Modify..."));
+	gtk_label_set_mnemonic_widget (GTK_LABEL (program_chooser->details->status_label), change_button);
 
 	g_signal_connect_object (change_button, "clicked",
 				 G_CALLBACK (run_program_configurator_callback), program_chooser, 0);
@@ -1456,6 +1457,7 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
 
 	caption = gtk_label_new (_("You can configure which programs are offered "
 				   "for which file types in the File Types and Programs dialog."));
+	gtk_label_set_mnemonic_widget (GTK_LABEL (caption), capplet_button);
 	gtk_widget_show (caption);
 	gtk_label_set_line_wrap (GTK_LABEL (caption), TRUE);
 	gtk_box_pack_start (GTK_BOX (capplet_hbox), caption, FALSE, FALSE, 0);				    
@@ -1598,11 +1600,13 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	if (action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
 		unavailable_message = g_strdup_printf (_("No viewers are available for \"%s\"."), file_name);
 		dialog_title = g_strdup (_("No Viewers Available"));
-	} else {
-		g_assert (action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION);
+	} else if (action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
 		unavailable_message = g_strdup_printf (_("There is no application associated with \"%s\"."), file_name);
 		dialog_title = g_strdup (_("No Application Associated"));
-	}
+	} else {
+		unavailable_message = g_strdup_printf (_("There is no action associated with \"%s\"."), file_name);
+		dialog_title = g_strdup (_("No Action Associated"));
+	}		
 
 	/* Note: This might be misleading in the components case, since the
 	 * user can't add components to the complete list even from the capplet.
@@ -1610,15 +1614,59 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	 */
 	prompt = g_strdup_printf (_("%s\n\n"
 				    "You can configure GNOME to associate applications "
-				    "with file types.  Do you want to associate an "
+				    "with file types. Do you want to associate an "
 				    "application with this file type now?"),
 				  unavailable_message);
+	
 	dialog = eel_show_yes_no_dialog 
 		(prompt, dialog_title, _("Associate Application"), GTK_STOCK_CANCEL, parent_window);
-
+		
 	g_signal_connect_object (dialog, "response",
-		G_CALLBACK (launch_mime_capplet_on_ok),
-		file, 0);
+				 G_CALLBACK (launch_mime_capplet_on_ok),
+				 file, 0);
+
+	g_free (unavailable_message);
+	g_free (file_name);
+	g_free (prompt);
+	g_free (dialog_title);
+}
+
+void
+nautilus_program_chooser_show_invalid_message (GnomeVFSMimeActionType  action_type,
+					        NautilusFile           *file, 
+					        GtkWindow              *parent_window)
+{
+	char *prompt;
+	char *unavailable_message;
+	char *file_name;
+	char *dialog_title;
+	GtkDialog *dialog;
+
+	file_name = get_file_name_for_display (file);
+
+	if (action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
+		unavailable_message = g_strdup_printf (_("The viewer associated with \"%s\" is invalid."), file_name);
+		dialog_title = g_strdup (_("Invalid Viewer Associated"));
+	} else if (action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
+		unavailable_message = g_strdup_printf (_("The application associated with \"%s\" is invalid."), file_name);
+		dialog_title = g_strdup (_("Invalid Application Associated"));
+	} else {
+		unavailable_message = g_strdup_printf (_("The action associated with \"%s\" is invalid."), file_name);
+		dialog_title = g_strdup (_("Invalid Action Associated"));
+	}		
+
+	prompt = g_strdup_printf (_("%s\n\n"
+				    "You can configure GNOME to associate a different application "
+				    "or viewer with this file type. Do you want to associate an "
+				    "application or viewer with this file type now?"),
+				  unavailable_message);
+
+	dialog = eel_show_yes_no_dialog 
+		(prompt, dialog_title, _("Associate Action"), GTK_STOCK_CANCEL, parent_window);
+	
+	g_signal_connect_object (dialog, "response",
+	G_CALLBACK (launch_mime_capplet_on_ok),
+	file, 0);
 
 	g_free (unavailable_message);
 	g_free (file_name);
