@@ -63,6 +63,7 @@
 #include <libnautilus-private/nautilus-customization-data.h>
 #include <libnautilus-private/nautilus-entry.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
+#include <libnautilus-private/nautilus-desktop-icon-file.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-emblem-utils.h>
@@ -1764,8 +1765,7 @@ create_emblems_page (FMPropertiesWindow *window)
 	NautilusFile *file;
 	GList *icons, *l;
 
-	
-	file = window->details->target_file;
+	file = window->details->original_file;
 
 	/* The emblems wrapped table */
 	scroller = eel_scrolled_wrap_table_new (TRUE, &emblems_table);
@@ -2384,7 +2384,7 @@ static FMPropertiesWindow *
 create_properties_window (StartupData *startup_data)
 {
 	FMPropertiesWindow *window;
-	GList *attributes;
+	NautilusFileAttributes attributes;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *button;
@@ -2407,20 +2407,16 @@ create_properties_window (StartupData *startup_data)
 	 * target file.
 	 */
 	attributes = nautilus_icon_factory_get_required_file_attributes ();
-	attributes = g_list_prepend (attributes,
-				     NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME);
+	attributes |= NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME;
 	nautilus_file_monitor_add (window->details->original_file, window, attributes);
-	g_list_free (attributes);
 
-	attributes = NULL;
+	attributes = 0;
 	if (nautilus_file_is_directory (window->details->target_file)) {
-		attributes = g_list_prepend (attributes,
-					     NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS);
+		attributes |= NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS;
 	}
-	attributes = g_list_prepend (attributes,
-				     NAUTILUS_FILE_ATTRIBUTE_METADATA);
+	
+	attributes |= NAUTILUS_FILE_ATTRIBUTE_METADATA;
 	nautilus_file_monitor_add (window->details->target_file, window, attributes);
-	g_list_free (attributes);
 
 	/* React to future property changes and file deletions. */
 	window->details->file_changed_handler_id =
@@ -2484,34 +2480,21 @@ static NautilusFile *
 get_target_file (NautilusFile *file)
 {
 	NautilusFile *target_file;
-	char *uri;
 	char *uri_to_display;
+	NautilusDesktopLink *link;
 
 	target_file = NULL;
-	if (nautilus_file_is_nautilus_link (file)) {
-		/* Note: This will only work on local files. For now
-		 * that seems fine since the links we care about are
-		 * all on the desktop.
-		 */
-		if (nautilus_file_is_local (file)) {
-			uri = nautilus_file_get_uri (file);
-
-			switch (nautilus_link_local_get_link_type (uri, NULL)) {
-			case NAUTILUS_LINK_MOUNT:
-			case NAUTILUS_LINK_TRASH:
-			case NAUTILUS_LINK_HOME:
-				/* map to linked URI for these types of links */
-				uri_to_display = nautilus_link_local_get_link_uri (uri);
-				target_file = nautilus_file_get (uri_to_display);
-				g_free (uri_to_display);
-				break;
-			case NAUTILUS_LINK_GENERIC:
-				/* don't for these types */
-				break;
-			}
-			
-			g_free (uri);
+	if (NAUTILUS_IS_DESKTOP_ICON_FILE (file)) {
+		link = nautilus_desktop_icon_file_get_link (NAUTILUS_DESKTOP_ICON_FILE (file));
+		
+		/* map to linked URI for these types of links */
+		uri_to_display = nautilus_desktop_link_get_activation_uri (link);
+		if (uri_to_display) {
+			target_file = nautilus_file_get (uri_to_display);
+			g_free (uri_to_display);
 		}
+		
+		g_object_unref (link);
 	}
 
 	if (target_file != NULL) {
@@ -2591,7 +2574,6 @@ fm_properties_window_present (NautilusFile *original_file, FMDirectoryView *dire
 	GtkWidget *parent_window;
 	NautilusFile *target_file;
 	StartupData *startup_data;
-	GList attribute_list;
 
 	g_return_if_fail (NAUTILUS_IS_FILE (original_file));
 	g_return_if_fail (FM_IS_DIRECTORY_VIEW (directory_view));
@@ -2640,11 +2622,8 @@ fm_properties_window_present (NautilusFile *original_file, FMDirectoryView *dire
 		 _("Cancel Showing Properties Window?"),
 		 _("Creating Properties window"),
 		 parent_window == NULL ? NULL : GTK_WINDOW (parent_window));
-	attribute_list.data = NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY;
-	attribute_list.next = NULL;
-	attribute_list.prev = NULL;
 	nautilus_file_call_when_ready
-		(target_file, &attribute_list,
+		(target_file, NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY,
 		 create_properties_window_callback, startup_data);
 }
 
