@@ -27,12 +27,17 @@
 
 #include <config.h>
 #include "nautilus-bookmarks-window.h"
+#include "nautilus-window.h"
 #include <libnautilus/nautilus-undo.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <eel/eel-gtk-extensions.h>
+#include <eel/eel-gnome-extensions.h>
+#include <libegg/egg-screen-help.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-undo-signal-handlers.h>
+#include <gtk/gtkhbbox.h>
 #include <gnome.h>
+#include <glade/glade.h>
 
 
 /* Static variables to keep track of window state. If there were
@@ -43,17 +48,19 @@ static int		     bookmark_list_changed_signal_id;
 static NautilusBookmarkList *bookmarks = NULL;
 static GtkTreeView	    *bookmark_list_widget = NULL; /* awkward name to distinguish from NautilusBookmarkList */
 static GtkListStore	    *bookmark_list_store = NULL;
+static GtkListStore	    *bookmark_empty_list_store = NULL;
 static GtkTreeSelection     *bookmark_selection = NULL;
 static int                   selection_changed_id = 0;
 static GtkWidget	    *name_field = NULL;
 static int		     name_field_changed_signal_id;
 static GtkWidget	    *remove_button = NULL;
+static GtkWidget            *jump_button = NULL;
 static gboolean		     text_changed = FALSE;
 static GtkWidget	    *uri_field = NULL;
 static int		     uri_field_changed_signal_id;
 static int		     row_changed_signal_id;
 static int		     row_deleted_signal_id;
-
+static int                   row_activated_signal_id;
 
 /* forward declarations */
 static guint    get_selected_row                            (void);
@@ -66,6 +73,8 @@ static void     on_name_field_changed                       (GtkEditable
 							     gpointer              user_data);
 static void     on_remove_button_clicked                    (GtkButton            *button,
 							     gpointer              user_data);
+static void     on_jump_button_clicked                      (GtkButton            *button,
+							     gpointer              user_data);
 static void	on_row_changed				    (GtkListStore	  *store,
 							     GtkTreePath	  *path,
 							     GtkTreeIter	  *iter,
@@ -73,6 +82,10 @@ static void	on_row_changed				    (GtkListStore	  *store,
 static void	on_row_deleted				    (GtkListStore	  *store,
 							     GtkTreePath	  *path,
 							     gpointer		   user_data);
+static void	on_row_activated			    (GtkTreeView	  *view,
+							     GtkTreePath	  *path,
+                                                             GtkTreeViewColumn    *column,
+							     gpointer		   user_data);
 static void     on_selection_changed                        (GtkTreeSelection     *treeselection,
 							     gpointer              user_data);
 
@@ -99,7 +112,8 @@ static void     set_up_close_accelerator                    (GtkWidget
 #define BOOKMARK_LIST_COLUMN_ICON		0
 #define BOOKMARK_LIST_COLUMN_NAME		1
 #define BOOKMARK_LIST_COLUMN_BOOKMARK		2
-#define BOOKMARK_LIST_COLUMN_COUNT		3
+#define BOOKMARK_LIST_COLUMN_STYLE		3
+#define BOOKMARK_LIST_COLUMN_COUNT		4
 
 /* layout constants */
 
@@ -119,17 +133,17 @@ nautilus_bookmarks_window_response_callback (GtkDialog *dialog,
 	if (response_id == GTK_RESPONSE_HELP) {
 		GError *error = NULL;
 
-		gnome_help_display_desktop (NULL,
-					    "user-guide",
-					    "wgosnautilus.xml", "gosnautilus-36",
-					    &error);
+		egg_help_display_desktop_on_screen (
+			NULL, "user-guide", "wgosnautilus.xml", "gosnautilus-36",
+			gtk_window_get_screen (GTK_WINDOW (dialog)), &error);
+
 		if (error) {
 			GtkWidget *err_dialog;
 			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
 							     GTK_DIALOG_DESTROY_WITH_PARENT,
 							     GTK_MESSAGE_ERROR,
 							     GTK_BUTTONS_CLOSE,
-							     _("There was an error displaying help: %s"),
+							     _("There was an error displaying help: \n%s"),
 							     error->message);
 
 			g_signal_connect (G_OBJECT (err_dialog),
@@ -139,8 +153,58 @@ nautilus_bookmarks_window_response_callback (GtkDialog *dialog,
 			gtk_widget_show (err_dialog);
 			g_error_free (error);
 		}
-	} else
+	} else if (response_id == GTK_RESPONSE_CLOSE) {
 		gtk_widget_hide (GTK_WIDGET (dialog));
+        }
+}
+
+static GtkListStore *
+create_bookmark_store (void)
+{
+	return gtk_list_store_new (BOOKMARK_LIST_COLUMN_COUNT,
+				   GDK_TYPE_PIXBUF,
+				   G_TYPE_STRING,
+				   G_TYPE_OBJECT,
+				   PANGO_TYPE_STYLE);
+}
+
+static void
+setup_empty_list (void)
+{
+	GtkTreeIter iter;
+
+	bookmark_empty_list_store = create_bookmark_store ();
+	gtk_list_store_append (bookmark_empty_list_store, &iter);
+
+	gtk_list_store_set (bookmark_empty_list_store, &iter,
+			    BOOKMARK_LIST_COLUMN_NAME, _("No bookmarks defined"),
+			    BOOKMARK_LIST_COLUMN_STYLE, PANGO_STYLE_ITALIC,
+			    -1);
+}
+
+static void
+bookmarks_set_empty (gboolean empty)
+{
+	GtkTreeIter iter;
+
+	if (empty) {
+		gtk_tree_view_set_model (bookmark_list_widget,
+					 GTK_TREE_MODEL (bookmark_empty_list_store));
+		gtk_widget_set_sensitive (GTK_WIDGET (bookmark_list_widget), FALSE);
+	} else {
+		gtk_tree_view_set_model (bookmark_list_widget,
+					 GTK_TREE_MODEL (bookmark_list_store));
+		gtk_widget_set_sensitive (GTK_WIDGET (bookmark_list_widget), TRUE);
+
+		if (nautilus_bookmark_list_length (bookmarks) > 0 &&
+		    !get_selection_exists ()) {
+			gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (bookmark_list_store),
+						       &iter, NULL, 0);
+			gtk_tree_selection_select_iter (bookmark_selection, &iter);
+		}
+	}
+
+	on_selection_changed (bookmark_selection, NULL);
 }
 
 /**
@@ -155,45 +219,30 @@ GtkWindow *
 create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_source)
 {
 	GtkWidget         *window;
-	GtkWidget         *content_area;
-	GtkWidget         *list_scroller;
-	GtkWidget         *right_side;
-	GtkWidget         *vbox3;
-	GtkWidget         *name_label;
-	GtkWidget         *vbox4;
-	GtkWidget         *url_label;
-	GtkWidget         *hbox2;
 	GtkTreeViewColumn *col;
 	GtkCellRenderer   *rend;
+	GladeXML          *gui;
 
 	bookmarks = list;
 
-	window = gtk_dialog_new_with_buttons (_("Bookmarks"), NULL, 0,
-					      GTK_STOCK_HELP, GTK_RESPONSE_HELP,
-					      GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
-					      NULL);
-	gtk_dialog_set_has_separator (GTK_DIALOG (window), FALSE);
+	gui = eel_glade_get_file (GLADEDIR "/nautilus-bookmarks-window.glade",
+				  NULL, NULL,
+				  "bookmarks_dialog", &window,
+				  "bookmark_tree_view", &bookmark_list_widget,
+				  "bookmark_delete_button", &remove_button,
+                                  "bookmark_jump_button", &jump_button,
+				  NULL);
+	if (!gui) {
+		return NULL;
+	}
+
 	set_up_close_accelerator (window);
 	nautilus_undo_share_undo_manager (G_OBJECT (window), undo_manager_source);
+
 	gtk_window_set_wmclass (GTK_WINDOW (window), "bookmarks", "Nautilus");
 	nautilus_bookmarks_window_restore_geometry (window);
 	
-	content_area = gtk_hbox_new (TRUE, GNOME_PAD);
-	gtk_widget_show (content_area);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), content_area, TRUE, TRUE, 0);
-
-	list_scroller = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_show (list_scroller);
-	gtk_box_pack_start (GTK_BOX (content_area), list_scroller, TRUE, TRUE, 0);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (list_scroller), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (list_scroller), GTK_SHADOW_IN);
-
-	bookmark_list_widget = GTK_TREE_VIEW (gtk_tree_view_new ());
-	gtk_widget_show (GTK_WIDGET (bookmark_list_widget));
-	gtk_container_add (GTK_CONTAINER (list_scroller), 
-			   GTK_WIDGET (bookmark_list_widget));
-	gtk_tree_view_set_headers_visible (bookmark_list_widget, FALSE);
-	gtk_tree_view_set_reorderable (bookmark_list_widget, TRUE);
+	bookmark_list_widget = GTK_TREE_VIEW (glade_xml_get_widget (gui, "bookmark_tree_view"));
 		
 	rend = gtk_cell_renderer_pixbuf_new ();
 	col = gtk_tree_view_column_new_with_attributes ("Icon", 
@@ -211,63 +260,39 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 							rend,
 							"text", 
 							BOOKMARK_LIST_COLUMN_NAME,
+							"style",
+							BOOKMARK_LIST_COLUMN_STYLE,
 							NULL);
 	gtk_tree_view_append_column (bookmark_list_widget,
 				     GTK_TREE_VIEW_COLUMN (col));
 	
-	bookmark_list_store = gtk_list_store_new (BOOKMARK_LIST_COLUMN_COUNT,
-						  GDK_TYPE_PIXBUF,
-						  G_TYPE_STRING,
-						  G_TYPE_INT,
-						  G_TYPE_OBJECT);
+	bookmark_list_store = create_bookmark_store ();
+	setup_empty_list ();
 	gtk_tree_view_set_model (bookmark_list_widget,
-				 GTK_TREE_MODEL (bookmark_list_store));
+				 GTK_TREE_MODEL (bookmark_empty_list_store));
 	
 	bookmark_selection =
 		GTK_TREE_SELECTION (gtk_tree_view_get_selection (bookmark_list_widget));
-	
-	right_side = gtk_vbox_new (FALSE, GNOME_PAD);
-	gtk_widget_show (right_side);
-	gtk_box_pack_start (GTK_BOX (content_area), right_side, TRUE, TRUE, 0);
-
-	vbox3 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox3);
-	gtk_box_pack_start (GTK_BOX (right_side), vbox3, FALSE, FALSE, 0);
-
-	name_label = gtk_label_new_with_mnemonic (_("_Name"));
-	gtk_widget_show (name_label);
-	gtk_box_pack_start (GTK_BOX (vbox3), name_label, FALSE, FALSE, 0);
 
 	name_field = nautilus_entry_new ();
 	gtk_widget_show (name_field);
-	gtk_box_pack_start (GTK_BOX (vbox3), name_field, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (glade_xml_get_widget (gui, "bookmark_name_placeholder")),
+			    name_field, TRUE, TRUE, 0);
 	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (name_field), TRUE);
 	
-	gtk_label_set_mnemonic_widget (GTK_LABEL (name_label), name_field);
-
-	vbox4 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox4);
-	gtk_box_pack_start (GTK_BOX (right_side), vbox4, FALSE, FALSE, 0);
-
-	url_label = gtk_label_new_with_mnemonic (_("_Location"));
-	gtk_widget_show (url_label);
-	gtk_box_pack_start (GTK_BOX (vbox4), url_label, FALSE, FALSE, 0);
+	gtk_label_set_mnemonic_widget (
+		GTK_LABEL (glade_xml_get_widget (gui, "bookmark_name_label")),
+		name_field);
 
 	uri_field = nautilus_entry_new ();
 	gtk_widget_show (uri_field);
-	gtk_box_pack_start (GTK_BOX (vbox4), uri_field, FALSE, FALSE, 0);	
+	gtk_box_pack_start (GTK_BOX (glade_xml_get_widget (gui, "bookmark_location_placeholder")),
+			    uri_field, TRUE, TRUE, 0);
 	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (uri_field), TRUE);
 
-	gtk_label_set_mnemonic_widget (GTK_LABEL (url_label), uri_field);
-
-	hbox2 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox2);
-	gtk_box_pack_start (GTK_BOX (right_side), hbox2, FALSE, FALSE, 0);
-
-	remove_button = gtk_button_new_with_mnemonic (_("_Remove"));
-
-	gtk_widget_show (remove_button);
-	gtk_box_pack_start (GTK_BOX (hbox2), remove_button, TRUE, FALSE, 0);
+	gtk_label_set_mnemonic_widget (
+		GTK_LABEL (glade_xml_get_widget (gui, "bookmark_location_label")),
+		uri_field);
 
 	bookmark_list_changed_signal_id =
 		g_signal_connect (bookmarks, "contents_changed",
@@ -278,6 +303,9 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 	row_deleted_signal_id =
 		g_signal_connect (bookmark_list_store, "row_deleted",
 				  G_CALLBACK (on_row_deleted), NULL);
+        row_activated_signal_id =
+                g_signal_connect (bookmark_list_widget, "row_activated",
+                                  G_CALLBACK (on_row_activated), undo_manager_source);
 	selection_changed_id =
 		g_signal_connect (bookmark_selection, "changed",
 				  G_CALLBACK (on_selection_changed), NULL);	
@@ -310,6 +338,8 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 			  G_CALLBACK (name_or_uri_field_activate), NULL);
 	g_signal_connect (remove_button, "clicked",
 			  G_CALLBACK (on_remove_button_clicked), NULL);
+	g_signal_connect (jump_button, "clicked",
+			  G_CALLBACK (on_jump_button_clicked), undo_manager_source);
 
 	/* Register to find out about icon theme changes */
 	g_signal_connect_object (nautilus_icon_factory_get (), "icons_changed",
@@ -321,6 +351,8 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 	/* Fill in list widget with bookmarks, must be after signals are wired up. */
 	repopulate();
 
+	g_object_unref (G_OBJECT (gui));
+	
 	return GTK_WINDOW (window);
 }
 
@@ -332,6 +364,9 @@ get_selected_bookmark (void)
 	if (!get_selection_exists())
 		return NULL;
 
+	if (nautilus_bookmark_list_length (bookmarks) < 1)
+		return NULL;
+
 	return nautilus_bookmark_list_item_at(bookmarks, get_selected_row ());
 }
 
@@ -376,7 +411,7 @@ nautilus_bookmarks_window_restore_geometry (GtkWidget *window)
 	if (window_geometry != NULL) {	
 		eel_gtk_window_set_initial_geometry_from_string 
 			(GTK_WINDOW (window), window_geometry, 
-			 BOOKMARKS_WINDOW_MIN_WIDTH, BOOKMARKS_WINDOW_MIN_HEIGHT);
+			 BOOKMARKS_WINDOW_MIN_WIDTH, BOOKMARKS_WINDOW_MIN_HEIGHT, FALSE);
 
 	} else {
 		/* use default since there was no stored geometry */
@@ -445,6 +480,27 @@ on_name_field_changed (GtkEditable *editable,
 	text_changed = TRUE;
 }
 
+static void
+go_to_selected_bookmark (NautilusWindow *window)
+{
+        NautilusBookmark *selected;
+        char *uri = NULL;
+
+        selected = get_selected_bookmark ();
+
+        if (selected) {
+                uri = nautilus_bookmark_get_uri (selected);
+                nautilus_window_go_to (window, uri);
+                g_free (uri);
+        }
+}
+
+static void
+on_jump_button_clicked (GtkButton *button,
+                        gpointer   user_data)
+{
+        go_to_selected_bookmark (NAUTILUS_WINDOW (user_data));
+}
 
 static void
 on_remove_button_clicked (GtkButton *button,
@@ -474,9 +530,13 @@ on_remove_button_clicked (GtkButton *button,
 	if (row >= rows)
 		row = rows - 1;
 
-	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (bookmark_list_store),
-				       &iter, NULL, row);
-	gtk_tree_selection_select_iter (bookmark_selection, &iter);
+	if (row < 0) {
+		bookmarks_set_empty (TRUE);
+	} else {
+		gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (bookmark_list_store),
+					       &iter, NULL, row);
+		gtk_tree_selection_select_iter (bookmark_selection, &iter);
+	}
 }
 
 /* This is a bit of a kludge to get DnD to work. We check if the row in the
@@ -493,6 +553,8 @@ on_row_changed (GtkListStore *store,
 	gint *indices, row;
 	gboolean insert_bookmark = TRUE;
 
+	store = bookmark_list_store;
+
 	indices = gtk_tree_path_get_indices (path);
 	row = indices[0];
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
@@ -526,6 +588,15 @@ on_row_changed (GtkListStore *store,
 	}
 }
 
+static void
+on_row_activated (GtkTreeView       *view,
+                  GtkTreePath       *path,
+                  GtkTreeViewColumn *column,
+                  gpointer           user_data)
+{
+        go_to_selected_bookmark (NAUTILUS_WINDOW (user_data));
+}
+
 static void
 on_row_deleted (GtkListStore *store,
 		GtkTreePath *path,
@@ -560,6 +631,7 @@ on_selection_changed (GtkTreeSelection *treeselection,
 	
 	/* Set the sensitivity of widgets that require a selection */
 	gtk_widget_set_sensitive (remove_button, selected != NULL);
+        gtk_widget_set_sensitive (jump_button, selected != NULL);
 	gtk_widget_set_sensitive (name_field, selected != NULL);
 	gtk_widget_set_sensitive (uri_field, selected != NULL);
 
@@ -685,7 +757,7 @@ on_window_hide_event (GtkWidget *widget,
 	nautilus_undo_unregister (G_OBJECT (uri_field));
 
 	/* restore_geometry only works after window is hidden */
-	gtk_idle_add (restore_geometry, widget);
+	g_idle_add (restore_geometry, widget);
 }
 
 static void
@@ -693,16 +765,14 @@ on_window_destroy_event (GtkWidget *widget,
 		      	 gpointer user_data)
 {
 	g_message ("destroying bookmarks window");
-	gtk_idle_remove_by_data (widget);
+	g_source_remove_by_user_data (widget);
 }
 
 static void
 repopulate (void)
 {
+	guint index;
 	GtkListStore *store;
-	guint         index;
-	GtkTreeIter   iter;
-	gboolean      selection_exists;
 	
 	g_assert (GTK_IS_TREE_VIEW (bookmark_list_widget));
 	g_assert (NAUTILUS_IS_BOOKMARK_LIST (bookmarks));
@@ -713,9 +783,13 @@ repopulate (void)
 				selection_changed_id);
 	g_signal_handler_block (bookmark_list_store,
 				row_deleted_signal_id);
-	
+        g_signal_handler_block (bookmark_list_store,
+                                row_activated_signal_id);
+
 	gtk_list_store_clear (store);
 	
+	g_signal_handler_unblock (bookmark_selection,
+				  row_activated_signal_id);
 	g_signal_handler_unblock (bookmark_list_store,
 				  row_deleted_signal_id);
 	g_signal_handler_unblock (bookmark_selection,
@@ -724,7 +798,7 @@ repopulate (void)
 	/* Fill the list in with the bookmark names. */
 	g_signal_handler_block (store, row_changed_signal_id);
 
-	for (index = 0; index < nautilus_bookmark_list_length(bookmarks); ++index) {
+	for (index = 0; index < nautilus_bookmark_list_length (bookmarks); ++index) {
 		NautilusBookmark *bookmark;
 		char             *bookmark_name;
 		GdkPixbuf        *bookmark_pixbuf;
@@ -741,6 +815,7 @@ repopulate (void)
 				    BOOKMARK_LIST_COLUMN_ICON, bookmark_pixbuf,
 				    BOOKMARK_LIST_COLUMN_NAME, bookmark_name,
 				    BOOKMARK_LIST_COLUMN_BOOKMARK, bookmark,
+				    BOOKMARK_LIST_COLUMN_STYLE, PANGO_STYLE_NORMAL,
 				    -1);
 		
 		g_free (bookmark_name);
@@ -749,33 +824,7 @@ repopulate (void)
 	}
 	g_signal_handler_unblock (store, row_changed_signal_id);
 
-	/* Select the first row on start-up */
-	if (index) {
-		gtk_tree_model_get_iter_root (GTK_TREE_MODEL (bookmark_list_store), &iter);
-		gtk_tree_selection_select_iter (bookmark_selection, &iter);
-	}
-	
-	/* Set the sensitivity of widgets that require a selection */
-	selection_exists = get_selection_exists ();
-	gtk_widget_set_sensitive (remove_button, selection_exists);
-	gtk_widget_set_sensitive (name_field, selection_exists);
-	gtk_widget_set_sensitive (uri_field, selection_exists);
-	    
-	if (!selection_exists) {
-		/* Block signals to avoid modifying nonexistent selected item. */
-		g_signal_handler_block (name_field, 
-					name_field_changed_signal_id);
-		nautilus_entry_set_text (NAUTILUS_ENTRY (name_field), "");
-		g_signal_handler_unblock (name_field, 
-					  name_field_changed_signal_id);
-
-		g_signal_handler_block (uri_field, 
-					uri_field_changed_signal_id);
-		nautilus_entry_set_text (NAUTILUS_ENTRY (uri_field), "");
-		g_signal_handler_unblock (uri_field,
-					  uri_field_changed_signal_id);
-	}
-	  
+	bookmarks_set_empty (index == 0);	  
 }
 
 static int
